/*
 * Authors: Zsolt Molnar <zsolt.molnar@cern.ch, http://www.zsoltmolnar.hu>
 *
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Unit tests for Savannah bug #60334: [lcg_util] lcg-cp hangs when file has no
 * replica
 *
 * Test if the logic recognizes unknown protocols of wrongly formatted
 * protocol lists in the transfetParameters property of PrepareToGet.
 */
#include "lcg_util.h"
#include "gfal_api.h"
#include "gfal_dependencies.h"
#include "globus_dependencies.h"
#include <assert.h>
#include <errno.h>
#include <string.h>

#include "lcg_unit_testsuite.h"

#define GFAL_RETURN_CODE_OK 0
#define GFAL_RETURN_CODE_ERROR -1

static int _caller_play_scenario_gfal_turlsfromsurls(
    gfal_internal req, char *errbuf, int errbufsz)
{
    errno = 0;
    return GFAL_RETURN_CODE_OK;
}

static int _caller_play_scenario_gfal_ls(
    gfal_internal req, char *errbuf, int errbufsz)
{
    return GFAL_RETURN_CODE_OK;
}

static int _caller_play_scenario_gfal_get_results(
    gfal_internal req, gfal_filestatus **results)
{
    *results = malloc(sizeof(gfal_filestatus));
    (*results)[0].status = 0;
    (*results)[0].turl = malloc(sizeof(char));
    (*results)[0].turl[0]= 0;
    return 1;
}

static int _caller_play_scenario_gfal_set_xfer_done(
    gfal_internal gfal, char *errbuf, int errbufsz)
{
    errno = 0;
    return 0;
}

static int _caller_play_scenario_globus_copyfilex(
    char *src_file, char *dest_file, int nbstreams, char *errbuf,
    int errbufsz, int timeout, int verbose)
{
    errno = 0;
    return 0;
}

static int _caller_play_scenario_globus_getfilesizet(
    char *file, GFAL_LONG64 *size, char *errbuf, int errbufsz, int timeout)
{
    errno = 0;
    *size = 1;
    return 0;
}

static int _caller_play_scenario_gfal_register_file (
    const char *lfn, const char *guid, const char *surl, mode_t mode,
    GFAL_LONG64 size, int bool_createonly, char *errbuf, int errbufsz)
{
    errno = 0;
    return 0;
}

static char * _caller_play_scenario_gfal_guidforpfn (
    const char *pfn, char *errbuf, int errbufsz)
{
    static const char* v = "fake_guid";
    char *ret = malloc(strlen(v) + 1);
    return strcpy(ret, v);
}

/* Catch the nbreplicas setup, and force it to 0 */
static gfal_file _caller_play_scenario_gfal_file_new (
    const char *file, const char *defproto, int bool_tobecreated, char *errbuf,
    int errbufsz)
{
    gfal_file gf = gfal_file_new(file, defproto, bool_tobecreated, errbuf,
        errbufsz);

    gf->nbreplicas = 0;
    return gf;
}

/* Inject the dependencies. Replace the SOAP callers to the test ones. */
static void _setup_callers()
{
    lcg_gfal_functions.call_gfal_turlsfromsurls = _caller_play_scenario_gfal_turlsfromsurls;
    lcg_gfal_functions.call_gfal_ls = _caller_play_scenario_gfal_ls;
    lcg_gfal_functions.call_gfal_get_results = _caller_play_scenario_gfal_get_results;
    lcg_gfal_functions.call_gfal_set_xfer_done = _caller_play_scenario_gfal_set_xfer_done;
    lcg_gfal_functions.call_gfal_register_file = _caller_play_scenario_gfal_register_file;
    lcg_gfal_functions.call_gfal_guidforpfn = _caller_play_scenario_gfal_guidforpfn;
    lcg_gfal_functions.call_gfal_file_new = _caller_play_scenario_gfal_file_new;

    lcg_globus_functions.call_globus_copyfilex = _caller_play_scenario_globus_copyfilex;
    lcg_globus_functions.call_globus_getfilesizet = _caller_play_scenario_globus_getfilesizet;
}

static char *mock_src_file = "gsiftp://src";
static char *mock_src_file_srm = "srm://fake2.cern.ch:8443/srm/managerv2?SFN=/src";
static char *mock_dst_file_srm = "srm://fake.cern.ch:8443/srm/managerv2?SFN=/dst";
static char *mock_vo = "fake_vo";
static int mock_nbfiles = 0;
static int mock_nobdii = 1;
static const char *mock_srm_endpoint = "httpg://srm-v2.cern.ch:8443/srm/managerv2";
static int mock_desiredpintime = 10;
static char *mock_spacetokendesc = NULL;
static char ** mock_reqtoken = NULL;
static struct srmv2_pinfilestatus * mock_pinfilestatus[3];
static int mock_timeout = 100;
static int mock_nbstreams = 1;
static char *mock_conf_file = NULL;
static int mock_insecure = 1;
static int mock_verbose = 0;
static enum gfal_cksm_type mock_cksmtype = GFAL_CKSM_NONE;
static int errbufsz = 1024;
static char errbuf[1024];
static char *mock_relative_path = NULL;
static char **mock_actual_surl = NULL;
static char *mock_guid = "fake_guid";
static char *mock_actual_guid = NULL;
static char *mock_lfn_arg = NULL;

/**
 * The functionality is present in lcg_cp and lcg_rep only
 */
char * lcg_test__60334_lcg_cp_hangs_when_file_has_no_replica()
{
    int res = 1;
    _setup_callers();
    errno = 0;

    res = lcg_cp5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2,
        mock_nobdii, mock_vo, mock_nbstreams, mock_conf_file, mock_insecure,
        mock_verbose, mock_timeout, mock_spacetokendesc, mock_spacetokendesc,
        mock_cksmtype, NULL, NULL, errbuf, errbufsz);

    LCG_TEST_EQUAL(EINVAL, errno);
    LCG_TEST_EQUAL(GFAL_RETURN_CODE_ERROR, res);
    errno = 0;

    res = lcg_rep5 (mock_src_file_srm, mock_dst_file_srm, TYPE_SRMv2, TYPE_SRMv2,
        TYPE_SRMv2, mock_nobdii, mock_vo, mock_relative_path, mock_nbstreams,
        mock_conf_file, mock_insecure, mock_verbose, mock_timeout,
        mock_spacetokendesc, mock_spacetokendesc, mock_actual_surl,
        mock_cksmtype, NULL, NULL, errbuf, errbufsz);

    LCG_TEST_EQUAL(EINVAL, errno);
    LCG_TEST_EQUAL(GFAL_RETURN_CODE_ERROR, res);

    return 0x0;
}

#undef __CALL_TEST
